@@ -8,7 +8,7 @@ local info = {
 
 local P = lpeg.P
 
-local lexer          = require("lexer")
+local lexer          = require("scite-context-lexer")
 local context        = lexer.context
 local patterns       = context.patterns
 
