@@ -9,6 +9,7 @@ if not modules then modules = { } end modules ['mtx-cache'] = {
 local concat, sort, insert = table.concat, table.sort, table.insert
 local gsub, format, gmatch, find = string.gsub, string.format, string.gmatch, string.find
 local utfchar, utfgsub = utf.char, utf.gsub
+local sortedkeys = table.sortedkeys
 
 local helpinfo = [[
 <?xml version="1.0"?>
@@ -34,6 +35,8 @@ local helpinfo = [[
     <flag name="text"><short>create text files for commands and environments</short></flag>
     <flag name="raw"><short>report commands to the console</short></flag>
     <flag name="check"><short>generate check file</short></flag>
+    <flag name="meaning"><short>report the mening of commands</short></flag>
+    <flag name="tokens"><short>show the internal representation of commands</short></flag>
    </subcategory>
    <subcategory>
     <flag name="toutf"><short>replace named characters by utf</short></flag>
@@ -43,6 +46,9 @@ local helpinfo = [[
     <flag name="suffix"><short>use given suffix for output files</short></flag>
     <flag name="force"><short>force action even when in doubt</short></flag>
    </subcategory>
+   <subcategory>
+    <flag name="pattern"><short>a pattern for meaning lookups</short></flag>
+   </subcategory>
   </category>
  </flags>
 </application>
@@ -85,9 +91,31 @@ end
 
 function flushers.scite(collected)
     local data = { }
-    for interface, whatever in next, collected do
-        data[interface] = whatever.commands
+--     for interface, whatever in next, collected do
+--         data[interface] = whatever.commands
+--     end
+    local function add(target,origin,field)
+        if origin then
+            local list = origin[field]
+            if list then
+                for i=1,#list do
+                    target[list[i]] = true
+                end
+            end
+        end
     end
+    --
+    for interface, whatever in next, collected do
+        local combined = { }
+        add(combined,whatever,"commands")
+        add(combined,whatever,"environments")
+        if interface == "common" then
+            add(combined,whatever,"textnames")
+            add(combined,whatever,"mathnames")
+        end
+        data[interface] = sortedkeys(combined)
+    end
+    --
     collect("scite-context-data-interfaces", "context",  data)
     collect("scite-context-data-metapost",   "metapost", dofile(resolvers.findfile("mult-mps.lua")))
     collect("scite-context-data-metafun",    "metafun",  dofile(resolvers.findfile("mult-fun.lua")))
@@ -141,6 +169,10 @@ function flushers.raw(collected)
     end
 end
 
+function flushers.data(collected)
+    return collected -- table.save("cont-en-filenames.lua",collected.filenames)
+end
+
 local textpadcreator = "mtx-interface-textpad.lua"
 
 function flushers.text(collected)
@@ -226,167 +258,17 @@ function flushers.textpad(collected)
     end
 end
 
--- function scripts.interface.editor(editor,split,forcedinterfaces)
---     local interfaces= forcedinterfaces or environment.files
---     if #interfaces == 0 then
---         interfaces= userinterfaces
---     end
---     local xmlfile = resolvers.findfile("cont-en.xml") or ""
---     if xmlfile == "" then
---         report("unable to locate cont-en.xml")
---     end
---     local collected = { }
---     for i=1,#interfaces do
---         local interface = interfaces[i]
---         local keyfile = resolvers.findfile(format("keys-%s.xml",interface)) or ""
---         if keyfile == "" then
---             report("unable to locate keys-*.xml")
---         else
---             local commands     = { }
---             local mappings     = { }
---             local environments = { }
---             local x = xml.load(keyfile)
---             for e, d, k in xml.elements(x,"/cd:interface/cd:commands/cd:command") do -- somehow this was variable
---                 local at = d[k].at
---                 local name, value = at.name, at.value
---                 if name and value then
---                     mappings[name] = value
---                 end
---             end
---             local x = xml.load(xmlfile)
---             for e, d, k in xml.elements(x,"/cd:interface/cd:command") do
---                 local at = d[k].at
---                 local name, type = at.name, at["type"]
---                 if name and name ~= "" then
---                     local remapped = mappings[name] or name
---                     if type == "environment" then
---                         if split then
---                             environments[#environments+1] = remapped
---                         else
---                             commands[#commands+1] = "start" .. remapped
---                             commands[#commands+1] = "stop"  .. remapped
---                         end
---                     else
---                         commands[#commands+1] = remapped
---                     end
---                 end
---             end
---             if #commands > 0 then
---                 sort(commands)
---                 sort(environments)
---                 collected[interface] = {
---                     commands     = commands,
---                     environments = environments,
---                 }
---             end
---         end
---     end
---     -- awaiting completion of the xml file
---     local definitions = dofile(resolvers.findfile("mult-def.lua"))
---     if definitions then
---         local commands = { en = { } }
---         for command, languages in next, definitions.commands do
---             commands.en[languages.en or command] = true
---             for language, command in next, languages do
---                 local c = commands[language]
---                 if c then
---                     c[command] = true
---                 else
---                     commands[language] = { [command] = true }
---                 end
---             end
---         end
---         for language, data in next, commands do
---             local fromlua = data
---             local fromxml = collected[language].commands
---             for i=1,#fromxml do
---                 local c = fromxml[i]
---                 if not fromlua[c] then
---                  -- print(language,c)
---                     fromlua[c] = true
---                 end
---             end
---             collected[language].commands = table.sortedkeys(fromlua)
---         end
---     end
---     --
---     flushers[editor](collected)
--- end
-
--- function scripts.interface.editor(editor,split,forcedinterfaces)
---     local interfaces= forcedinterfaces or environment.files
---     if #interfaces == 0 then
---         interfaces= userinterfaces
---     end
---     --
---     local filename = "context-en.xml"
---     local xmlfile  = resolvers.findfile(filename) or ""
---     if xmlfile == "" then
---         report("unable to locate %a",filename)
---         return
---     end
---     local x = xml.load(xmlfile)
---     --
---     local filename = "mult-def.lua"
---     local deffile  = resolvers.findfile(filename) or ""
---     if deffile == "" then
---         report("unable to locate %a",filename)
---         return
---     end
---     local interface = dofile(filename)
---     if not interface or not next(interface) then
---         report("invalid file %a",filename)
---         return
---     end
---     local variables = interface.variables
---     local constants = interface.constants
---     local commands  = interface.commands
---     local elements  = interface.elements
---     local collected = { }
---     for i=1,#interfaces do
---         local interface      = interfaces[i]
---         local i_commands     = { }
---         local i_environments = { }
---         local start = elements.start[interface] or elements.start.en
---         local stop  = elements.stop [interface] or elements.stop .en
---         for e, d, k in xml.elements(x,"cd:interface/cd:command") do
---             local at   = d[k].at
---             local name = at["name"] or ""
---             local type = at["type"]
---             if name ~= "" then
---                 local c = commands[name]
---                 local n = c and (c[interface] or c.en) or name
---                 if type ~= "environment" then
---                     i_commands[#i_commands+1] = n
---                 elseif split then
---                     i_environments[#i_environments+1] = n
---                 else
---                     -- variables ?
---                     i_commands[#i_commands+1] = start .. n
---                     i_commands[#i_commands+1] = stop  .. n
---                 end
---             end
---         end
---         if #i_commands > 0 then
---             sort(i_commands)
---             sort(i_environments)
---             collected[interface] = {
---                 commands     = i_commands,
---                 environments = i_environments,
---             }
---         end
---     end
---     --
---     flushers[editor](collected)
--- end
-
 function scripts.interface.editor(editor,split,forcedinterfaces)
-    local interfaces= forcedinterfaces or environment.files
-    if #interfaces == 0 then
-        interfaces= userinterfaces
+    require("char-def")
+
+    local interfaces = forcedinterfaces or environment.files or userinterfaces
+    if not interfaces.en then
+        -- loaded as script so we have "cont-yes.*" as name
+        interfaces = { "en" }
     end
     --
-    local filename = "i-context.xml"
+ -- local filename = "i-context.xml"
+    local filename = "context-en.xml"
     local xmlfile  = resolvers.findfile(filename) or ""
     if xmlfile == "" then
         report("unable to locate %a",filename)
@@ -414,14 +296,34 @@ function scripts.interface.editor(editor,split,forcedinterfaces)
     report("generating files for %a",editor)
     report("loading %a",xmlfile)
     local xmlroot = xml.load(xmlfile)
-    xml.include(xmlroot,"cd:interfacefile","filename",true,function(s)
-        local fullname = resolvers.findfile(s)
-        if fullname and fullname ~= "" then
-            report("including %a",fullname)
-            return io.loaddata(fullname)
-        end
-    end)
-    --
+ -- xml.include(xmlroot,"cd:interfacefile","filename",true,function(s)
+ --     local fullname = resolvers.findfile(s)
+ --     if fullname and fullname ~= "" then
+ --         report("including %a",fullname)
+ --         return io.loaddata(fullname)
+ --     end
+ -- end)
+ -- local definitions = { }
+ -- for e in xml.collected(xmlroot,"cd:interface/cd:define") do
+ --     definitions[e.at.name] = e.dt
+ -- end
+ -- local function resolve(root)
+ --     for e in xml.collected(root,"*") do
+ --         if e.tg == "resolve" then
+ --             local resolved = definitions[e.at.name or ""]
+ --             if resolved then
+ --                 -- use proper replace helper
+ --                 e.__p__.dt[e.ni] = resolved
+ --                 resolved.__p__ = e.__p__
+ --                 resolve(resolved)
+ --             end
+ --         end
+ --     end
+ -- end
+ -- resolve(xmlroot)
+
+    -- todo: use sequence
+
     for i=1,#interfaces do
         local interface      = interfaces[i]
         local i_commands     = { }
@@ -435,22 +337,62 @@ function scripts.interface.editor(editor,split,forcedinterfaces)
             if name ~= "" then
                 local c = commands[name]
                 local n = c and (c[interface] or c.en) or name
+                local sequence = xml.all(e,"/cd:sequence/*")
                 if at.generated == "yes" then
+                    for e in xml.collected(e,"/cd:instances/cd:constant") do
+                        local name = e.at.value
+                        if name then
+                            local c = variables[name]
+                            local n = c and (c[interface] or c.en) or name
+                            if sequence then
+                                local done = { }
+                                for i=1,#sequence do
+                                    local e = sequence[i]
+                                    if e.tg == "string" then
+                                        local value = e.at.value
+                                        if value then
+                                            done[i] = value
+                                        else
+                                            done = false
+                                            break
+                                        end
+                                    else
+                                        local tg = e.tg
+                                        if tg == "instance" or tg == "instance:assignment" or tg == "instance:ownnumber" then
+                                            done[i] = name
+                                        else
+                                            done = false
+                                            break
+                                        end
+                                    end
+                                end
+                                if done then
+                                    n = concat(done)
+                                end
+                            end
+                            if type ~= "environment" then
+                                i_commands[n] = true
+                            elseif split then
+                                i_environments[n] = true
+                            else
+                                i_commands[start..n] = true
+                                i_commands[stop ..n] = true
+                            end
+                        end
+                    end
                     -- skip (for now)
                 elseif type ~= "environment" then
-                    i_commands[#i_commands+1] = n
+                    i_commands[n] = at.file or true
                 elseif split then
-                    i_environments[#i_environments+1] = n
+                    i_environments[n] = at.file or true
                 else
                     -- variables ?
-                    i_commands[#i_commands+1] = start .. n
-                    i_commands[#i_commands+1] = stop  .. n
+                    i_commands[start..n] = at.file or true
+                    i_commands[stop ..n] = at.file or true
                 end
             end
         end
-        if #i_commands > 0 then
-            sort(i_commands)
-            sort(i_environments)
+        if next(i_commands) then
             collected[interface] = {
                 commands     = i_commands,
                 environments = i_environments,
@@ -458,7 +400,92 @@ function scripts.interface.editor(editor,split,forcedinterfaces)
         end
     end
     --
-    flushers[editor](collected)
+    local commoncommands     = { }
+    local commonenvironments = { }
+    for k, v in next, collected do
+        local c = v.commands
+        local e = v.environments
+        if k == "en" then
+            for k, v in next, c do
+                commoncommands[k] = true
+            end
+            for k, v in next, e do
+                commonenvironments[k] = true
+            end
+        else
+            for k, v in next, c do
+                if not commoncommands[k] then
+                    commoncommands[k] = nil
+                end
+            end
+            for k, v in next, e do
+                if not commonenvironments[k] then
+                    commonenvironments[k] = nil
+                end
+            end
+        end
+    end
+    local filenames = { }
+    if collected.en then
+        for k, v in next, collected.en.commands do
+            if v ~= true then
+                filenames[k] = v
+            end
+        end
+        for k, v in next, collected.en.environments do
+            if v ~= true then
+                filenames[k] = v
+            end
+        end
+    end
+    for k, v in next, collected do
+        local c = v.commands
+        local e = v.environments
+        for k, v in next, commoncommands do
+            c[k] = nil
+        end
+        for k, v in next, commonenvironments do
+            e[k] = nil
+        end
+        v.commands     = sortedkeys(c)
+        v.environments = sortedkeys(e)
+    end
+    --
+    local mathnames = { }
+    local textnames = { }
+    for k, v in next, characters.data do
+        local name = v.contextname
+        if name then
+            textnames[name] = true
+        end
+        local name = v.mathname
+        if name then
+            mathnames[name] = true
+        end
+        local spec = v.mathspec
+        if spec then
+            for i=1,#spec do
+                local s = spec[i]
+                local name = s.name
+                if name then
+                    mathnames[name] = true
+                end
+            end
+        end
+    end
+    --
+    collected.common = {
+        textnames    = sortedkeys(textnames),
+        mathnames    = sortedkeys(mathnames),
+        commands     = sortedkeys(commoncommands),
+        environments = sortedkeys(commonenvironments),
+        filenames    = filenames,
+    }
+    --
+    local flusher = flushers[editor]
+    if flusher then
+        return flusher(collected)
+    end
 end
 
 function scripts.interface.check()
@@ -500,7 +527,7 @@ function scripts.interface.mkii()
                 texresult[#texresult+1] = format("%% definitions for interface %s for language %s\n%%",what,language)
                 xmlresult[#xmlresult+1] = format("\t<!-- definitions for interface %s for language %s -->\n",what,language)
                 xmlresult[#xmlresult+1] = format("\t<cd:%s>",what)
-                local sorted = table.sortedkeys(t)
+                local sorted = sortedkeys(t)
                 for i=1,#sorted do
                     local key = sorted[i]
                     local v = t[key]
@@ -570,8 +597,8 @@ function scripts.interface.mkii()
 end
 
 function scripts.interface.preprocess()
-    dofile(resolvers.findfile("luat-mac.lua"))
- -- require("luat-mac.lua")
+    require("luat-mac")
+
     local newsuffix = environment.argument("suffix") or "log"
     local force = environment.argument("force")
     for i=1,#environment.files do
@@ -588,10 +615,45 @@ function scripts.interface.preprocess()
     end
 end
 
+function scripts.interface.bidi()
+    require("char-def")
+
+    local directiondata  = { }
+    local mirrordata     = { }
+    local textclassdata  = { }
+
+    local data = {
+        comment    = "generated by: mtxrun -- script interface.lua --bidi",
+        direction  = directions,
+        mirror     = mirrors,
+        textclass  = textclasses,
+    }
+
+    for k, d in next, characters.data do
+        local direction = d.direction
+        local mirror    = d.mirror
+        local textclass = d.textclass
+        if direction and direction ~= "l" then
+            directiondata[k] = direction
+        end
+        if mirror then
+            mirrordata[k] = mirror
+        end
+        if textclass then
+            textclassdata[k] = textclass
+        end
+    end
+
+    local filename = "scite-context-data-bidi.lua"
+
+    report("saving %a",filename)
+    table.save(filename,data)
+end
+
 function scripts.interface.toutf()
     local filename = environment.files[1]
     if filename then
-        require("char-def.lua")
+        require("char-def")
         local contextnames = { }
         for unicode, data in next, characters.data do
             local contextname = data.contextname
@@ -605,7 +667,7 @@ function scripts.interface.toutf()
             contextnames.aumlaut = contextnames.adiaeresis
             contextnames.Aumlaut = contextnames.Adiaeresis
         end
-        report("loading '%s'",filename)
+        report("loading %a",filename)
         local str = io.loaddata(filename) or ""
         local done = { }
         str = gsub(str,"(\\)([a-zA-Z][a-zA-Z][a-zA-Z]+)(%s*)", function(b,s,a)
@@ -626,19 +688,65 @@ function scripts.interface.toutf()
             end
         end
         filename = filename .. ".toutf"
-        report("saving '%s'",filename)
+        report("saving %a",filename)
         io.savedata(filename,str)
     end
 end
 
+function scripts.interface.meaning()
+    local runner  = "mtxrun --silent --script context --extra=meaning --once --noconsole --nostatistics"
+    local pattern = environment.arguments.pattern
+    local files   = environment.files
+    if type(pattern) == "string" then
+        runner = runner .. ' --pattern="' .. pattern .. '"'
+    elseif files and #files > 0 then
+        for i=1,#files do
+            runner = runner .. ' "' .. files[i] .. '"'
+        end
+    else
+        return
+    end
+    local r = os.resultof(runner)
+    if type(r) == "string" then
+        r = gsub(r,"^.-(meaning%s+>)","\n%1")
+        print(r)
+    end
+end
+
+function scripts.interface.tokens()
+    local runner  = "mtxrun --silent --script context --extra=meaning --tokens --once --noconsole --nostatistics"
+    local pattern = environment.arguments.pattern
+    local files   = environment.files
+    if type(pattern) == "string" then
+        runner = runner .. ' --pattern="' .. pattern .. '"'
+    elseif files and #files > 0 then
+        for i=1,#files do
+            runner = runner .. ' "' .. files[i] .. '"'
+        end
+    else
+        return
+    end
+    local r = os.resultof(runner)
+    if type(r) == "string" then
+        r = gsub(r,"^.-(tokens%s+>)","\n%1")
+        print(r)
+    end
+end
+
 local ea = environment.argument
 
 if ea("mkii") then
     scripts.interface.mkii()
 elseif ea("preprocess") then
     scripts.interface.preprocess()
+elseif ea("meaning") then
+    scripts.interface.meaning()
+elseif ea("tokens") then
+    scripts.interface.tokens()
 elseif ea("toutf") then
     scripts.interface.toutf()
+elseif ea("bidi") then
+    scripts.interface.bidi()
 elseif ea("check") then
     scripts.interface.check()
 elseif ea("scite") or ea("bbedit") or ea("jedit") or ea("textpad") or ea("text") or ea("raw") then
