@@ -22,6 +22,7 @@ local rep, sub = string.rep, string.sub
 local setmetatable = setmetatable
 local filesuffix = file.suffix
 local convertlmxstring = lmx and lmx.convertstring
+local savedata = io.savedata
 
 local pushtarget, poptarget = logs.pushtarget, logs.poptarget
 
@@ -29,36 +30,125 @@ local report_macros = logs.reporter("interface","macros")
 
 local stack, top, n, hashes = { }, nil, 0, { }
 
-local function set(s)
-    if top then
-        n = n + 1
-        if n > 9 then
-            report_macros("number of arguments > 9, ignoring %s",s)
-        else
+-- local function set(s)
+--     if top then
+--         n = n + 1
+--         if n > 9 then
+--             report_macros("number of arguments > 9, ignoring %s",s)
+--         else
+--             local ns = #stack
+--             local h = hashes[ns]
+--             if not h then
+--                 h = rep("#",2^(ns-1))
+--                 hashes[ns] = h
+--             end
+--             if s == "ignore" then
+--                 top[s] = ""
+--                 return h .. "0"
+--             else
+--                 local m = h .. n
+--                 top[s] = m
+--                 return m
+--             end
+--         end
+--     end
+-- end
+
+-- local function get(s)
+--     if s == "ignore" then
+--         return ""
+--     else
+--         if not top then
+--             report_macros("keeping #%s, no stack",s)
+--             return "#" .. s -- can be lua
+--         end
+--         local m = top[s]
+--         if m then
+--             return m
+--         else
+--             report_macros("keeping #%s, not on stack",s)
+--             return "#" .. s -- quite likely an error
+--         end
+--     end
+-- end
+
+local set = CONTEXTLMTXMODE > 0 and
+    function(s)
+        if top then
             local ns = #stack
             local h = hashes[ns]
             if not h then
                 h = rep("#",2^(ns-1))
                 hashes[ns] = h
             end
-            local m = h .. n
-            top[s] = m
-            return m
+            if s == "ignore" then
+                return h .. "-"
+            elseif s == "spacer" then
+                return h .. "*"
+            elseif s == "pickup" then
+                return h .. ":"
+            else
+                n = n + 1
+                if n > 9 then
+                    report_macros("number of arguments > 9, ignoring %s",s)
+                elseif s == "discard" then
+                    top[s] = ""
+                    return h .. "0"
+                elseif s == "keepbraces" then
+                    top[s] = ""
+                    return h .. "+"
+                elseif s == "mandate" then
+                    top[s] = ""
+                    return h .. "="
+                elseif s == "keepmandate" then
+                    top[s] = ""
+                    return h .. "_"
+                elseif s == "prunespacing" then
+                    top[s] = ""
+                    return h .. "/"
+                else
+                    local m = h .. n
+                    top[s] = m
+                    return m
+                end
+            end
+        end
+    end
+or
+    function(s)
+        if top then
+            local ns = #stack
+            local h = hashes[ns]
+            if not h then
+                h = rep("#",2^(ns-1))
+                hashes[ns] = h
+            end
+            n = n + 1
+            if n > 9 then
+                report_macros("number of arguments > 9, ignoring %s",s)
+            else
+                local m = h .. n
+                top[s] = m
+                return m
+            end
         end
     end
-end
 
 local function get(s)
-    if not top then
-        report_macros("keeping #%s, no stack",s)
-        return "#" .. s -- can be lua
-    end
-    local m = top[s]
-    if m then
-        return m
+    if s == "ignore" or s == "discard" then
+        return ""
     else
-        report_macros("keeping #%s, not on stack",s)
-        return "#" .. s -- quite likely an error
+        if not top then
+            report_macros("keeping #%s, no stack",s)
+            return "#" .. s -- can be lua
+        end
+        local m = top[s]
+        if m then
+            return m
+        else
+            report_macros("keeping #%s, not on stack",s)
+            return "#" .. s -- quite likely an error
+        end
     end
 end
 
@@ -195,35 +285,27 @@ local checker = P("%") * (1 - newline - P("macros"))^0
 
 -- maybe namespace
 
-local macros = { } resolvers.macros = macros
+local resolvers  = resolvers
+
+local macros     = { }
+resolvers.macros = macros
+
+local loadtexfile = resolvers.loadtexfile
 
 function macros.preprocessed(str,strip)
     return lpegmatch(parser,str,1,strip)
 end
 
 function macros.convertfile(oldname,newname) -- beware, no testing on oldname == newname
-    local data = resolvers.loadtexfile(oldname)
-    data = interfaces.preprocessed(data) or "" -- interfaces not yet defined
-    io.savedata(newname,data)
+    local data = loadtexfile(oldname)
+    data = macros.preprocessed(data) or "" -- interfaces not yet defined
+    savedata(newname,data)
 end
 
 function macros.version(data)
     return lpegmatch(checker,data)
 end
 
--- function macros.processmkvi(str,filename)
---     if filename and filesuffix(filename) == "mkvi" or lpegmatch(checker,str) == "mkvi" then
---         local oldsize = #str
---         str = lpegmatch(parser,str,1,true) or str
---         pushtarget("log")
---         report_macros("processed mkvi file %a, delta %s",filename,oldsize-#str)
---         poptarget("log")
---     end
---     return str
--- end
---
--- utilities.sequencers.appendaction(resolvers.openers.helpers.textfileactions,"system","resolvers.macros.processmkvi")
-
 -- the document variables hack is temporary
 
 local processors = { }
@@ -231,9 +313,9 @@ local processors = { }
 function processors.mkvi(str,filename)
     local oldsize = #str
     str = lpegmatch(parser,str,1,true) or str
-    pushtarget("log")
+    pushtarget("logfile")
     report_macros("processed mkvi file %a, delta %s",filename,oldsize-#str)
-    poptarget("log")
+    poptarget()
     return str
 end
 
@@ -246,9 +328,9 @@ function processors.mkix(str,filename) -- we could intercept earlier so that cac
     end
     local oldsize = #str
     str = convertlmxstring(str,document.variables,false) or str
-    pushtarget("log")
+    pushtarget("logfile")
     report_macros("processed mkix file %a, delta %s",filename,oldsize-#str)
-    poptarget("log")
+    poptarget()
     return str
 end
 
@@ -262,12 +344,15 @@ function processors.mkxi(str,filename)
     local oldsize = #str
     str = convertlmxstring(str,document.variables,false) or str
     str = lpegmatch(parser,str,1,true) or str
-    pushtarget("log")
+    pushtarget("logfile")
     report_macros("processed mkxi file %a, delta %s",filename,oldsize-#str)
-    poptarget("log")
+    poptarget()
     return str
 end
 
+processors.mklx = processors.mkvi
+processors.mkxl = processors.mkiv
+
 function macros.processmk(str,filename)
     if filename then
         local suffix = filesuffix(filename)
@@ -279,50 +364,56 @@ function macros.processmk(str,filename)
     return str
 end
 
+local function validvi(filename,str)
+    local suffix = filesuffix(filename)
+    if suffix == "mkvi" or suffix == "mklx" then
+        return true
+    else
+        local check = lpegmatch(checker,str)
+        return check == "mkvi" or check == "mklx"
+    end
+end
+
 function macros.processmkvi(str,filename)
-    if filename and filesuffix(filename) == "mkvi" or lpegmatch(checker,str) == "mkvi" then
+    if filename and filename ~= "" and validvi(filename,str) then
         local oldsize = #str
         str = lpegmatch(parser,str,1,true) or str
-        pushtarget("log")
+        pushtarget("logfile")
         report_macros("processed mkvi file %a, delta %s",filename,oldsize-#str)
-        poptarget("log")
+        poptarget()
     end
     return str
 end
 
-local sequencers = utilities.sequencers
-
-if sequencers then
-
-    sequencers.appendaction(resolvers.openers.helpers.textfileactions,"system","resolvers.macros.processmk")
-    sequencers.appendaction(resolvers.openers.helpers.textfileactions,"system","resolvers.macros.processmkvi")
-
-end
+macros.processmklx = macros.processmkvi
 
 -- bonus
 
-if resolvers.schemes then
+local schemes = resolvers.schemes
+
+if schemes then
 
     local function handler(protocol,name,cachename)
         local hashed = url.hashed(name)
         local path = hashed.path
         if path and path ~= "" then
-            local str = resolvers.loadtexfile(path)
-            if filesuffix(path) == "mkvi" or lpegmatch(checker,str) == "mkvi" then
+            local str = loadtexfile(path)
+            if validvi(path,str) then
                 -- already done automatically
-                io.savedata(cachename,str)
+                savedata(cachename,str)
             else
                 local result = lpegmatch(parser,str,1,true) or str
-                pushtarget("log")
+                pushtarget("logfile")
                 report_macros("processed scheme %a, delta %s",filename,#str-#result)
-                poptarget("log")
-                io.savedata(cachename,result)
+                poptarget()
+                savedata(cachename,result)
             end
         end
         return cachename
     end
 
-    resolvers.schemes.install('mkvi',handler,1) -- this will cache !
+    schemes.install('mkvi',handler,1)
+    schemes.install('mklx',handler,1)
 
 end
 
@@ -334,6 +425,8 @@ end
 -- ]]))
 
 -- print(macros.preprocessed([[\checked \def \bla #bla{bla#{bla}}]]))
+-- print(macros.preprocessed([[\checked \def \bla #bla#discard#foo{bla#{bla}+#ignore+bla#foo}]]))
+-- print(macros.preprocessed([[\checked \def \bla #bla#ignore#foo{bla#{bla}+#ignore+bla#foo}]]))
 -- print(macros.preprocessed([[\def\bla#bla{#{bla}bla}]]))
 -- print(macros.preprocessed([[\def\blä#{blá}{blà:#{blá}}]]))
 -- print(macros.preprocessed([[\def\blä#bla{blà:#bla}]]))
@@ -414,7 +507,7 @@ local encodepattern = Cs((encodecomment + 1)^0)
 local decodecomment = P(commentsignal) / "%%%%" -- why doubles here?
 local decodepattern = Cs((decodecomment + 1)^0)
 
-function resolvers.macros.encodecomment(str)
+function macros.encodecomment(str)
     if txtcatcodes and tex.catcodetable == txtcatcodes then
         return lpegmatch(encodepattern,str) or str
     else
@@ -422,7 +515,7 @@ function resolvers.macros.encodecomment(str)
     end
 end
 
-function resolvers.macros.decodecomment(str) -- normally not needed
+function macros.decodecomment(str) -- normally not needed
     return txtcatcodes and lpegmatch(decodepattern,str) or str
 end
 
@@ -430,9 +523,22 @@ end
 -- resolvers.macros.encodecommentpattern = encodepattern
 -- resolvers.macros.decodecommentpattern = decodepattern
 
-function resolvers.macros.enablecomment(thecatcodes)
-    if not txtcatcodes then
-        txtcatcodes = thecatcodes or catcodes.numbers.txtcatcodes
-        utilities.sequencers.appendaction(resolvers.openers.helpers.textlineactions,"system","resolvers.macros.encodecomment")
+local sequencers   = utilities.sequencers
+local appendaction = sequencers and sequencers.appendaction
+
+if appendaction then
+
+    local textlineactions = resolvers.openers.helpers.textlineactions
+    local textfileactions = resolvers.openers.helpers.textfileactions
+
+    appendaction(textfileactions,"system","resolvers.macros.processmk")
+    appendaction(textfileactions,"system","resolvers.macros.processmkvi")
+
+    function macros.enablecomment(thecatcodes)
+        if not txtcatcodes then
+            txtcatcodes = thecatcodes or catcodes.numbers.txtcatcodes
+            appendaction(textlineactions,"system","resolvers.macros.encodecomment")
+        end
     end
+
 end
