@@ -920,6 +920,15 @@ local function handlenewline()
     currentline = currentline + 1
 end
 
+-- first = ":" | [A-Z] | "_" | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#x00F8-#x02FF] |
+--         [#x0370-#x037D] | [#x037F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] |
+--         [#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] |
+--         [#x10000-#xEFFFF]
+-- rest  = "-" | "." | [0-9] | #xB7 | [#x300-#x36F] | [#x203F-#x2040]
+-- name  = first + (first + rest)^1
+--
+-- We assume utf and do no real checking!
+
 local spacetab         = S(' \t')
 local space            = S(' \r\n\t')
 local newline          = lpegpatterns.newline / handlenewline
@@ -933,7 +942,11 @@ local slash            = P('/')
 local colon            = P(':')
 local semicolon        = P(';')
 local ampersand        = P('&')
-local valid            = R('az', 'AZ', '09') + S('_-.')
+----- valid_0          = lpegpatterns.utf8two + lpegpatterns.utf8three + lpegpatterns.utf8four
+local valid_0          = R("\128\255") -- basically any encoding without checking (fast)
+local valid_1          = R('az', 'AZ') + S('_') + valid_0
+local valid_2          = valid_1 + R('09') + S('-.')
+local valid            = valid_1 * valid_2^0
 local name_yes         = C(valid^1) * colon * C(valid^1)
 local name_nop         = C(P(true)) * C(valid^1)
 local name             = name_yes + name_nop
@@ -971,8 +984,9 @@ end
 local function entityfile(pattern,k,v,n)
     if n then
         local okay, data
-        if resolvers then
-            okay, data = resolvers.loadbinfile(n)
+        local loadbinfile = resolvers and resolvers.loadbinfile
+        if loadbinfile then
+            okay, data = loadbinfile(n)
         else
             data = io.loaddata(n)
             okay = data and data ~= ""
@@ -1146,13 +1160,15 @@ publicentityfile +
 
 end
 
-grammar_parsed_text_one_nop ,
-grammar_parsed_text_two_nop ,
-grammar_unparsed_text_nop   = install(space, spacing, anything)
+local
+    grammar_parsed_text_one_nop ,
+    grammar_parsed_text_two_nop ,
+    grammar_unparsed_text_nop   = install(space, spacing, anything)
 
-grammar_parsed_text_one_yes ,
-grammar_parsed_text_two_yes ,
-grammar_unparsed_text_yes   = install(space_nl, spacing_nl, anything_nl)
+local
+    grammar_parsed_text_one_yes ,
+    grammar_parsed_text_two_yes ,
+    grammar_unparsed_text_yes   = install(space_nl, spacing_nl, anything_nl)
 
 -- maybe we will add settings to result as well
 
@@ -1276,13 +1292,26 @@ end
 
 xml.convert = xmlconvert
 
-function xml.inheritedconvert(data,xmldata) -- xmldata is parent
+function xml.inheritedconvert(data,xmldata,cleanup) -- xmldata is parent
     local settings = xmldata.settings
     if settings then
         settings.parent_root = xmldata -- to be tested
     end
  -- settings.no_root = true
     local xc = xmlconvert(data,settings) -- hm, we might need to locate settings
+    if cleanup then
+        local x = xc.dt
+        if x then
+            x = x[1]
+            if x and x.tg == "@pi@" then
+                local dt = x.dt
+                local pi = dt and dt[1]
+                if type(pi) == "string" and find(pi,"^xml") then
+                    remove(dt,1)
+                end
+            end
+        end
+    end
  -- xc.settings = nil
  -- xc.entities = nil
  -- xc.special = nil
@@ -1398,9 +1427,17 @@ local function copy(old,p)
                 new[k] = t
             elseif k == "dt" then
                 v.__p__ = nil
-                v = copy(v,new)
-                new[k] = v
-                v.__p__ = p
+                local t = { }
+                for i=1,#v do
+                    local vi = v[i]
+                    if type(vi) == "table" then
+                        t[i] = copy(vi,new)
+                    else
+                        t[i] = vi
+                    end
+                end
+                new[k] = t
+                t.__p__ = p
             else
                 new[k] = v -- so we also share entities, etc in root
             end
